'use strict';

/**
 * @providesModule ReactImage
 */
import PropTypes from 'prop-types';

import React from 'react';
import createReactClass from 'create-react-class';
import View from '../View/View';
import { Mixin as LayoutMixin } from '../Utilties/LayoutMixin';
import ImageResizeMode from './ImageResizeMode';
import { Mixin as NativeMethodsMixin } from '../Utilties/NativeMethodsMixin';
import mixin from 'react-mixin';

const Image = createReactClass({
    displayName: 'Image',
    mixins: [LayoutMixin, NativeMethodsMixin],
    propTypes: {
        ...View.propTypes,
        /**
         * `uri` is a string representing the resource identifier for the image, which
         * could be an http address, a local file path, or a static image
         * resource (which should be wrapped in the `require('./path/to/image.png')` function).
         * This prop can also contain several remote `uri`, specified together with
         * their width and height. The native side will then choose the best `uri` to display
         * based on the measured size of the image container.
         */
        source: PropTypes.oneOfType([
            PropTypes.shape({
                uri: PropTypes.string,
            }),
            // Opaque type returned by require('./image.jpg')
            PropTypes.string,
        ]),
        /**
         * similarly to `source`, this property represents the resource used to render
         * the loading indicator for the image, displayed until image is ready to be
         * displayed, typically after when it got downloaded from network.
         */
        loadingIndicatorSource: PropTypes.oneOfType([
            PropTypes.shape({
                uri: PropTypes.string,
            }),
            // Opaque type returned by require('./image.jpg')
            PropTypes.number,
        ]),
        /**
         * Invoked on load start
         */
        onLoadStart: PropTypes.func,
        /**
         * Invoked when load completes successfully
         */
        onLoad: PropTypes.func,
        /**
         * Invoked when load either succeeds or fails
         */
        onLoadEnd: PropTypes.func,
        /**
         * Determines how to resize the image when the frame doesn't match the raw
         * image dimensions.
         *
         * 'cover': Scale the image uniformly (maintain the image's aspect ratio)
         * so that both dimensions (width and height) of the image will be equal
         * to or larger than the corresponding dimension of the view (minus padding).
         *
         * 'contain': Scale the image uniformly (maintain the image's aspect ratio)
         * so that both dimensions (width and height) of the image will be equal to
         * or less than the corresponding dimension of the view (minus padding).
         *
         * 'stretch': Scale width and height independently, This may change the
         * aspect ratio of the src.
         *
         * 'center': Scale the image down so that it is completely visible,
         * if bigger than the area of the view.
         * The image will not be scaled up.
         */
        resizeMode: PropTypes.oneOf(['cover', 'contain', 'stretch']),
    },

    statics: {
        resizeMode: ImageResizeMode,
        isReactNativeComponent: true
    },

    contextTypes: {
        isInAParentText: PropTypes.bool
    },

    getInitialState() {
        return {
            loadSuccess: false,
        };
    },

    UNSAFE_componentWillReceiveProps(nextProps) {
        let currSrc = typeof this.props.source === 'string' ? this.props.source : this.props.source.uri;
        let nextSrc = typeof nextProps.source === 'string' ? nextProps.source : nextProps.source.uri;
        if (currSrc !== nextSrc) {
            this._onLoadStart();
        }
    },

    render() {

        let props = {...this.props};
        props.src = typeof props.source === 'string' ? props.source : props.source.uri;

        // TODO: lazyload image when not in viewport

        let resizeMode = this.props.resizeMode || (this.props.style && this.props.style.resizeMode);

        // Background image element, resizeMode is strtch is equal default img style
        if ( (this.props.children || (resizeMode && resizeMode !== 'stretch')) && !this.context.isInAParentText) {
            let containerStyles = props.style ? props.style : {};
            containerStyles.backgroundImage = 'url("' + props.src + '")';
            containerStyles.backgroundSize = resizeMode || 'cover';
            containerStyles.backgroundRepeat = 'no-repeat';
            containerStyles.backgroundPosition = '50%';

            return (
                <View style={containerStyles} data-src={props.src} className={props.className}>
                    {!this.state.loadSuccess && <img src={props.src} onLoad={this._onLoad} onError={this._onError} style={{display: 'none'}} />}
                    {this.props.children}
                </View>
            );
        } else {
            let {...newProps} = props;
            let type = props.type;
            var style = newProps.style;
            if(type === 'gray_big' || type === 'gray_banner' || type === 'white_big'
                || type === 'white_banner' || type === 'gray_middle' || type === 'white_middle'
                || type === 'gray_small' || type === 'gray_logo'
                || type === 'white_logo' || type === 'white_small'){
                // style.opacity = 0;
            }

            return (
                !this.state.loadSuccess ?  <img {...props} onLoad={this._onLoad} onError={this._onError} style={style} />
                    : <img {...props} />
            );
        }
    },

    _onLoadStart() {
        this.setState({
            loadSuccess: false,
        });
        this.props.onLoadStart && this.props.onLoadStart();
    },

    _onLoad() {
        this.setState({
            loadSuccess: true,
        });
        this.props.onLoad && this.props.onLoad();
    },

    _onError() {
        this.props.onError && this.props.onError();
    },

    updateSource(url){

    }
});
Image.getSize = function(url, success, failure) {
    let img = document.createElement('img');
    img.src=url;
    img.onload = function(){
        success && success(this.width, this.height);

    };
    img.onerror = function () {
        failure && failure();
    }
};

export default Image;
